# Coronavirus 20 sec hand wash timer - clap controlled
# Adafruit CircuitPlayground Express
# Tony Goodhew 5th September 2020

# The Sound sensor (MEMS microphone) on the CPX is harder to use than that on the
# CPB so you need this complicated code to identify the loud sound of a hand clap
# It was copied from:
#   https://learn.adafruit.com/adafruit-circuit-playground-express/playground-sound-meter

# The MIT License (MIT)
#
# Copyright (c) 2017 Dan Halbert for Adafruit Industries
# Copyright (c) 2017 Kattni Rembor, Tony DiCola for Adafruit Industries
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# 20 sec hand wash timer - clap controlled

import array
import math
import audiobusio
import board
import time
from adafruit_circuitplayground import cp

# === MAIN ======= COPIED CODE =======================================
# Exponential scaling factor.
# Should probably be in range -10 .. 10 to be reasonable.
CURVE = 2
SCALE_EXPONENT = math.pow(10, CURVE * -0.1)

# Number of samples to read at once.
NUM_SAMPLES = 160

# Restrict value to be between floor and ceiling.
def constrain(value, floor, ceiling):
    return max(floor, min(value, ceiling))

# Scale input_value between output_min and output_max, exponentially.
def log_scale(input_value, input_min, input_max, output_min, output_max):
    normalized_input_value = (input_value - input_min) / \
                             (input_max - input_min)
    return output_min + \
        math.pow(normalized_input_value, SCALE_EXPONENT) \
        * (output_max - output_min)

# Remove DC bias before computing RMS.
def normalized_rms(values):
    minbuf = int(mean(values))
    samples_sum = sum(
        float(sample - minbuf) * (sample - minbuf)
        for sample in values
    )

    return math.sqrt(samples_sum / len(values))

def mean(values):
    return sum(values) / len(values)

# ====== My routines ============
def count():
    for i in range(10):
        cp.pixels[i] = (100,0,0) #Red
    cp.pixels.show()
    time.sleep(1.0)

    for i in range(10):
        cp.pixels[i] = (100,100,0) # Yellow
        cp.pixels.show()
        cp.play_tone(262, 0.1) # Tick sound
        time.sleep(0.9)
    for i in range(10):
        cp.pixels[i] = (0,127,0) # Green
        cp.pixels.show()
        cp.play_tone(262, 0.1)
        if not (i == 9):
            time.sleep(0.9)

    for i in range(6):
        cp.play_tone(262*2, 0.1)
        time.sleep(0.3)
    time.sleep(0.5)
    for i in range(10):
        cp.pixels[i] = (0,0,0)
    cp.pixels.show()

# Main program
cp.pixels.brightness = 0.01
# === COPIED CODE ==========
mic = audiobusio.PDMIn(board.MICROPHONE_CLOCK, board.MICROPHONE_DATA,
                       sample_rate=16000, bit_depth=16)

# Record an initial sample to calibrate. Assume it's quiet when we start.
samples = array.array('H', [0] * NUM_SAMPLES)
mic.record(samples, len(samples))
# Set lowest level to expect, plus a little.
input_floor = normalized_rms(samples) + 10
# OR: used a fixed floor
# input_floor = 50

# You might want to print the input_floor to help adjust other values.
# print(input_floor)

# Corresponds to sensitivity: lower means more pixels light up with lower sound
# Adjust this as you see fit.
input_ceiling = input_floor + 500

while True:
    mic.record(samples, len(samples))
    magnitude = normalized_rms(samples)
    # You might want to print this to see the values.
    print(magnitude)
# ========== END OF COPIED CODE =======================
    if magnitude > 750: # Check for hand clap - adjust value to suit environment
        count()